<?php
/**
 * ./classes/PHPTAL/Fib.php
 *
 * @package default
 */


require_once "PHPTAL/Tokenizer.php";

/**
 * any/any/any
 *
 * obj/method()
 *
 * obj/method()/methodresult
 *
 * obj/method(arg)
 *
 * array[]/keyliteral
 *
 * array[keyvalue]
 *
 * 100/50  # division
 *
 * val/50  # array key lookup
 *
 */
class Val
{


    /**
     *
     *
     * @param unknown $val
     * @param object  $e1
     * @param unknown $op
     * @param object  $e2  (optional)
     * @param unknown $str
     */
    function __construct($val)
    {
        $this->val = $val;
    }





    /**
     *
     *
     * @return unknown
     */
    function __toString()
    {
        return '('.$this->val.')';
    }


}


class TalesVal extends Val
{


    /**
     *
     *
     * @param unknown $val
     * @param object  $e1
     * @param unknown $op
     * @param object  $e2  (optional)
     * @param unknown $str
     */
    function __construct($val)
    {
        $this->val = $val;
    }





    /**
     *
     *
     * @return unknown
     */
    function __toString()
    {
        return phptal_tales($this->val);
    }


}


class Expr extends Val
{


    /**
     *
     *
     * @param unknown $val
     * @param object  $e1
     * @param unknown $op
     * @param object  $e2  (optional)
     * @param unknown $str
     */
    function __construct(Val $e1, $op, Val $e2 = NULL)
    {
        $this->e1 = $e1; $this->e2 = $e2; $this->op = $op;
    }





    /**
     *
     *
     * @return unknown
     */
    function __toString()
    {
        return '('.$this->e1.' '.$this->op.' '.$this->e2.')';
    }


}


/**
 *
 *
 * @param unknown $op
 * @return unknown
 */
function precedence($op)
{
    if ($op == '*' || $op == '/' || $op == '%') return 2;
    if ($op == '+' || $op == '-') return 1;
}


/**
 *
 *
 * @param object  $fib
 * @param object  $lhs
 * @param unknown $min_precedence
 * @return unknown
 */
function parse_expression(PHPTAL_Tokenizer $fib, Val $lhs, $min_precedence)
{
    $fib->skipSpace();

    while (($fib->token() == 'OPER' || $fib->token() == 'MINUS'))// && precedence($fib->tokenValue()) >= $min_precedence)
        {
        $op = $fib->tokenValue();
        $fib->nextToken();

        $rhs = parse_primary($fib);
        while (($fib->token() == 'OPER' || $fib->token() == 'MINUS') && precedence($fib->tokenValue()) > precedence($op)) // or a right-associative operator whose precedence is equal to op's
            {
            $lookahead = $fib->token();
            $rhs = parse_expression($fib, $rhs, precedence($lookahead));
        }
        $lhs = new Expr($lhs, $op, $rhs);

        $fib->skipSpace();
    }
    return $lhs;
}


/**
 *
 *
 * @param object  $fib
 * @return unknown
 */
function parse_primary(PHPTAL_Tokenizer $fib)
{
    $fib->skipSpace();
    //echo "primary ",$fib->token(),"\n";
    switch ($fib->token()) {
    case "NUM":
        $val = new Val($fib->tokenValue());
        $fib->nextToken();
        return $val;

    case "VAR":
    case "PATH":
        $val = new TalesVal($fib->tokenValue());
        $fib->nextToken();
        return $val;

    case "LPAREN":
        $fib->nextToken();
        $val = parse_expression($fib, parse_primary($fib), 0);
        if ("RPAREN" !== $fib->token()) throw new Exception("Unexpected ".$fib->token()."(".$fib->tokenValue().") instead of RPAREN");
        $fib->nextToken();
        return $val;

    case "MINUS":
        $fib->nextToken();
        return new Expr(parse_primary($fib), '-');

    default:
        throw new Exception("Unexpected ".$fib->token()."(".$fib->tokenValue().") in parse_primary");
    }
}


/**
 *
 *
 * @param unknown $expr
 * @param unknown $nothrow
 * @return unknown
 */
function phptal_tales_fib($expr, $nothrow)
{
    $fib = new FibTokenizer($expr);
    $fib->nextToken();

    $expr = parse_expression($fib, parse_primary($fib), 0);

    $fib->skipSpace();
    if (!$fib->eof()) throw new Exception("Unexpected ".$fib->token()."(".$fib->tokenValue().") instead of EOF");

    return $expr;
}


class PHPTAL_FibTokenizer extends PHPTAL_Tokenizer
{


    /**
     *
     *
     * @param unknown $val
     * @param object  $e1
     * @param unknown $op
     * @param object  $e2  (optional)
     * @param unknown $str
     */
    function __construct($str)
    {
        parent::__construct($str, array(
                'NUM'=>'\d+(?:\.\d+)?(?![a-z])', // decimal not followed by letters, screw exponent
                'MINUS'=>'-', // unary and binary
                'PATH'=>'[a-z][^:;\s*%+-/]*(?:/(?:\d+|[a-z][^:;\s*%+-/]*))+',
                'OPER'=>'[*%+]|(?<![a-z])/(?![a-z])', // standard operators, except minus (above) and division must not be surrounded by letters
                'VAR'=>'[a-z][^:;\s*%+-/]*(?![/a-z])',
                'SPACE'=>'\s+',
                'LPAREN'=>'\(',
                'RPAREN'=>'\)',
            ));
    }


}
