<?php
/**
 * ./classes/PHPTAL/PreFilter/Compress.php
 *
 * @package default
 */


class PHPTAL_PreFilter_Compress extends PHPTAL_PreFilter
{


	/**
	 *
	 *
	 * @param unknown $txt
	 * @return unknown
	 */
	function filter($txt) {
		return strtr(str_replace("\r\n", "\n", $txt), "\r", "\n");
	}





	/**
	 *
	 *
	 * @param object  $root
	 */
	function filterDOM(PHPTAL_Dom_Element $root) {
		//        $root->normalizeSpace();
		$this->compressElement($root);
	}


	const MODE_BODY_BLOCK = 0;
	const MODE_BODY_INLINE = 0;
	const MODE_HEAD = 1;
	const MODE_PRE = 2;



	/**
	 *
	 *
	 * @param object  $element
	 * @param unknown $d       (optional)
	 */
	private function compressElement(PHPTAL_Dom_Element $element, $d=0) {
		// don't compress tal:block attributes, because they won't be visible anyway
		if (!$element->getXmlnsState()->isHandledNamespace($element->getNamespaceURI())) {
			$this->compressAttributes($element);
		}

		// skip elements with xml:space="preserve", <pre>, etc.
		if ($this->isSpacePreserving($element)) {

			// HTML 5 (9.1.2.5) specifies quirk that a first *single* newline in <pre> can be removed
			if ($this->isHTMLElement($element, array('pre', 'textarea'))) {
				if (count($element->childNodes)) {
					$firstChild = $element->childNodes[0];
					if ($firstChild instanceof PHPTAL_Dom_Text || $firstChild instanceof PHPTAL_Dom_CDATASection) {
						$val = $firstChild->getValueEscaped();
						$firstChild->setValueEscaped(preg_replace('/^\n(?!\n)/', '', $val));
					}
				}
			}
			return;
		}

		echo "\n", str_repeat(' ', $d), "B4: "; foreach ($element->childNodes as $n) echo $n, ";";

		$following_node = $element->parentNode;
		if ($following_node && count($following_node->childNodes) && $following_node->childNodes[0] instanceof PHPTAL_Dom_Element) {
			$following_node = $following_node->childNodes[0];
		}

		$preceeding_node = $element->parentNode;

		if (count($element->childNodes)) {
			$childNodesCopy = $element->childNodes; // use copy, because removeChild changes indexes
			$i=0;
			$node = $preceeding_node;
			$nextnode = $childNodesCopy[0];
			do {
				$prevnode = $node;
				$node = $nextnode;
				$nextnode = (++$i < count($childNodesCopy)) ? $childNodesCopy[$i] : $following_node;

				echo "\nStrip: $prevnode + ($node) + $nextnode\n";

				if ($node instanceof PHPTAL_Dom_Element) {
					$this->compressElement($node, $d+1);
				}
				else if ($node instanceof PHPTAL_Dom_Text || $node instanceof PHPTAL_Dom_CDATASection) {
						$val = preg_replace('/\s+/', ' ', $node->getValueEscaped());
						if (!$this->isInlineElement($prevnode)) {
							$val = ltrim($val);
						}
						else {echo "\nnot stripped $node, a4 $prevnode\n";}
						if (!$this->isInlineElement($nextnode)) {
							$val = rtrim($val);
						}
						else {echo "\nnot stripped $node, b4 $prevnode\n";}

						if ($val === '') {
							$node->parentNode->removeChild($node);
							//$node = $prevnode;
						} else {
							$node->setValueEscaped($val);
						}
					}

			} while ($i < count($childNodesCopy));
		}

		echo "\n", str_repeat(' ', $d), "AD: "; foreach ($element->childNodes as $n) echo $n, ";";

	}



	/**
	 *
	 *
	 * @param object  $element
	 * @param array   $names
	 * @return unknown
	 */
	private function isHTMLElement(PHPTAL_Dom_Element $element, array $names) {
		$ns = $element->getNamespaceURI();

		return ($ns === 'http://www.w3.org/1999/xhtml' || $ns === '')
			&& in_array($element->getLocalName(), $names);
	}



	/**
	 *
	 *
	 * @param object  $element
	 * @return unknown
	 */
	private function isSpacePreserving(PHPTAL_Dom_Element $element) {
		return $this->isHTMLElement($element, array('pre', 'textarea'))
			|| $element->getAttributeNS("http://www.w3.org/XML/1998/namespace", 'space') === 'preserve';
	}



	/**
	 *
	 *
	 * @param object  $node
	 * @return unknown
	 */
	private function isInlineElement(PHPTAL_Dom_Node $node) {
		if (!$node instanceof PHPTAL_Dom_Element) {
			return true;
		}

		return $this->isHTMLElement($node, array('a', 'span', 'i', 'b', 'li'));
	}



	/**
	 *
	 *
	 * @param object  $element
	 */
	private function compressAttributes(PHPTAL_Dom_Element $element) {
		$attrs_by_qname = array();

		foreach ($element->getAttributeNodes() as $attrnode) {
			// skip attributes that are in a namespace (XHTML attributes have no namespace!)
			// skip replaced attributes (because getValueEscaped on them is meaningless)
			if ($attrnode->getNamespaceURI() !== ''
				|| $attrnode->getReplacedState() !== PHPTAL_Dom_Attr::NOT_REPLACED) {
				continue;
			}

			$val = $attrnode->getValueEscaped();
			$val = preg_replace($attrnode->getEncoding()==='UTF-8' ? '/\s+/u' : '/\s+/', ' ', trim($val));

			$attrnode->setValueEscaped($val);

			$attrs_by_qname[$attrnode->getQualifiedName()] = $attrnode; // there can't be two attrs with same qname
		}

		// sorting of attributes might give slightly better gzip performance
		if (count($attrs_by_qname) > 1) {
			uksort($attrs_by_qname, array($this, 'compareQNames'));
			$element->setAttributeNodes(array_values($attrs_by_qname));
		}
	}



	/**
	 * pre-defined order of attributes roughly by popularity
	 */
	private static $attributes_order = array('href', 'src', 'class', 'title', 'width', 'height', 'name', 'style', 'lang', 'id', 'rel');



	/**
	 * compare names according to $attributes_order array.
	 * Elements that are not in array, are considered greater than all elements in array,
	 * and are sorted alphabetically.
	 *
	 * @param unknown $a
	 * @param unknown $b
	 * @return unknown
	 */
	private function compareQNames($a, $b) {
		$a_index = array_search($a, self::$attributes_order);
		$b_index = array_search($b, self::$attributes_order);

		if ($a !== false && $b !== false) {
			return $a - $b;
		}
		if ($a === false && $b === false) {
			return strcmp($a, $b);
		}
		return ($a === false) ? 1 : -1;
	}


}
